@@ -6,6 +6,9 @@
 #
 #---------------------------------------------#
 import datetime
+import logging
+
+logger = logging.getLogger(__name__)
 
 from django.db import models
 from django.core.mail import EmailMultiAlternatives
@@ -13,8 +16,7 @@ from django.db.models.signals import post_save
 from django.dispatch import receiver
 from django.conf import settings
 
-from mailqueue import defaults
-
+from . import defaults
 
 class MailerMessageManager(models.Manager):
     def send_queued(self, limit=None):
@@ -24,7 +26,6 @@ class MailerMessageManager(models.Manager):
         for email in self.filter(sent=False)[:limit]:
             email.send()
 
-
 class MailerMessage(models.Model):
     subject = models.CharField(max_length=250, blank=True, null=True)
     to_address = models.EmailField(max_length=250)
@@ -41,6 +42,23 @@ class MailerMessage(models.Model):
     def __unicode__(self):
         return self.subject
 
+    def add_attachment(self, attachment):
+        """
+        Takes a Django `File` object and creates an attachment for this mailer message.
+        """
+        if self.pk is None:
+            self._save_without_sending()
+
+        Attachment.objects.create(email=self, file_attachment=attachment)
+
+    def _save_without_sending(self, *args, **kwargs):
+        """
+        Saves the MailerMessage instance without sending the e-mail. This ensures
+        other models (e.g. `Attachment`) have something to relate to in the database.
+        """
+        self.do_not_send = True
+        super(MailerMessage, self).save(*args, **kwargs)
+
     def send(self):
         if not self.sent:
             if getattr(settings, 'USE_TZ', False):
@@ -49,6 +67,7 @@ class MailerMessage(models.Model):
                 self.last_attempt = datetime.datetime.utcnow().replace(tzinfo=utc)
             else:
                 self.last_attempt = datetime.datetime.now()
+
             try:
                 subject, from_email, to = self.subject, self.from_address, self.to_address
                 text_content = self.content
@@ -61,17 +80,33 @@ class MailerMessage(models.Model):
                         msg.bcc = [ email.strip() for email in self.bcc_address.split(',') ]
                     else:
                         msg.bcc = [self.bcc_address, ]
+
+                # Add any additional attachments
+                for attachment in self.attachment_set.all():
+                    msg.attach_file(attachment.file_attachment.name)
+
                 msg.send()
                 self.sent = True
             except Exception, e:
-                print("mail queue exception %s" % e)
+                logger.error('Mail Queue Exception: {0}'.format(e))
+
             self.save()
 
+class Attachment(models.Model):
+    file_attachment = models.FileField(upload_to='mail-queue/attachments', blank=True, null=True)
+    email = models.ForeignKey(MailerMessage, blank=True, null=True)
+
+    def __unicode__(self):
+        return self.file_attachment.name
 
 @receiver(post_save, sender=MailerMessage)
 def send_post_save(sender, instance, signal, *args, **kwargs):
+    if getattr(instance, "do_not_send", False):
+        instance.do_not_send = False
+        return
+
     if getattr(settings, 'MAILQUEUE_CELERY', defaults.MAILQUEUE_CELERY):
         from mailqueue.tasks import send_mail
-        send_mail.delay(instance)
+        send_mail.delay(instance.pk)
     else:
         instance.send()
